$( document ).ready( () => {
  // link tab buttons to page switching
  $( '.tablinks' ).click( ( e ) => {
    $( e.currentTarget ).siblings().removeClass( "active" );
    $( e.currentTarget ).addClass( "active" );
    $( ".tab" ).hide();
    $( "#" + $( e.currentTarget ).data( 'tab' ) ).show();
  } );

  // link checkbox changes to group enable/disable
  $( 'input[type="checkbox"]' ).on( "change.groups", ( e ) => {
    $( "input." + $( e.currentTarget ).data( 'group' ) ).prop( 'disabled',
      $( e.currentTarget ).data( 'invert' )
        ? $( e.currentTarget ).prop( 'checked' )
        : !$( e.currentTarget ).prop( 'checked' )
    );
  } );

  // link config changes with device setting
  $( 'input, select' ).not(".noSend").on( "change.send", ( e ) => {
    if ( $( e.currentTarget ).is( ":invalid" ) ) return;
    let key = $( e.currentTarget ).prop( 'id' );
    let value = ( $( e.currentTarget ).prop( 'type' ) == 'checkbox' )
      ? c_bool( $( e.currentTarget ).prop( 'checked' ) )
      : $( e.currentTarget ).val();
    setConfig( key, value );
  } );

  // hide the reboot to apply button
  $("#reboot_to_apply").hide();

  // pre-select the first tab
  $( "#tabs button:first-child" ).click();

  // update status every 10 seconds
  setInterval( getStatus, 10000 );

  // get initial data
  getStatus();
  getConfig();
} );


// arduino boolean values
var c_bool = bool => bool ? 1 : 0;


// send config changes to device
function setConfig( key, value ) {
  $.ajax( {
    type: "post",
    url: "/api/set",
    data: {
      "key": key,
      "value": value
    },
    statusCode: {
      204: () => {
        // window.status = "Value already set.";
      },
      205: () => {
        // window.status = "Configuration accepted.";
        $("#reboot_to_apply").show();
      },
      403: xhr => {
        // window.status = "Configuration rejected.";
        unpackJson( xhr.responseJSON );
      }
    }
  } );
}


// retrieve configuration data from device
function getConfig() {
  $.ajax( {
    url: "/api/config",
    dataType: "json",
    success: data => unpackJson( data, "config" )
  } );
}


// retrieve status data from device
function getStatus() {
  $.ajax( {
    url: "/api/status",
    dataType: "json",
    success: data => unpackJson( data, "status" )
  } );
}


// fill UI with data from JSON query
function unpackJson( data, scope = false ) {
  scope = scope ? scope + "_" : "";
  $.each( data, ( key, value ) => {
    let id = scope + key;
    if ( isObject( value ) ) {
      // recursively unpack objects
      unpackJson( value, id );
    } else {
      //  set value of coresponding input
      let elmnt = $( "#" + id.toLowerCase() );
      if ( elmnt.is( 'input' ) || elmnt.is( 'select' ) ) {
        if ( elmnt.prop( 'type' ) == 'checkbox' ) {
          elmnt.prop( 'checked', ( value == "1" ) ? true : false );
          elmnt.trigger( "change.groups" );
        } else {
          elmnt.val( value );
        }
      } else {
        elmnt.text( value );
      }
    }
  } );
}


// toggle the password field to be readable
function showPassword( input, button ) {
  $( input ).prop( 'type',
    $( button ).prop( 'checked' )
    ? 'text'
    : 'password' );
}


// test if a variable is an object type
const isObject = ( obj ) => {
  return Object.prototype.toString.call( obj ) === '[object Object]';
};
