/*
  dmx_universe.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#ifndef DMX_UNIVERSE_H
#define DMX_UNIVERSE_H

#include <cstdint>
#include <functional>
#include <vector>


#define E111_NULL_START       0

// Table D1 - Reserved START Codes
#define E111_ASC_TEXT_ASCII   23
#define E111_ASC_TEST         85
#define E111_ASC_TEXT_UTF8    144
#define E111_ASC_MANUFACTURER 145
#define E111_ASC_SIP          207


// structure of DMX data
typedef union {
  struct {
    uint8_t  start_code;
    uint8_t  slot[512]; // 0 index-origin!
  } __attribute__((packed));
  uint8_t data[513];
} e111_universe_t;


// forward declare the Univserse class
class Universe;

// register type for data users to subscribe callbacks
typedef std::function<void(Universe *)> E111DataHandlerFunction;


/*
   The Universe class

   ANSI E1.11 describes may things about moving data over
   serial EIA-485-A links.  This class encapselates that data,
   regardless of how it was transmitted.
*/
class Universe {
  public:
    Universe();

    e111_universe_t * data() {
      return &data_;
    }
    bool setData(e111_universe_t *data);
    void onData(const E111DataHandlerFunction callback);
    uint32_t age(); // millis() since setData

  private:
    e111_universe_t data_;
    uint32_t time_;
    std::vector<E111DataHandlerFunction> callbacks_;
};

#endif // DMX_UNIVERSE_H
